/*************************************************************************\
* Copyright (c) 2006 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* This software is distributed subject to a Software License Agreement
* found in file LICENSE that is included with this distribution.
\*************************************************************************/

/*
 * MCF5282 CPU statistics
 */
#include <epicsStdio.h>
#include <epicsExport.h>
#include <cantProceed.h>
#include <asynDriver.h>
#include <asynInt32.h>

/*
 * Driver private storage
 */
typedef struct drvPvt {
    asynInterface common;
    asynInterface int32;
} drvPvt;

/*
 * asynCommon methods
 */
static void
report(void *pvt, FILE *fp, int details)
{
}

static asynStatus
connect(void *pvt, asynUser *pasynUser)
{
    pasynManager->exceptionConnect(pasynUser);
    return asynSuccess;
}

static asynStatus
disconnect(void *pvt, asynUser *pasynUser)
{
    pasynManager->exceptionDisconnect(pasynUser);
    return asynSuccess;
}
static asynCommon commonMethods = { report, connect, disconnect };

/*
 * int32 methods
 */
static asynStatus
int32Read(void *pvt, asynUser *pasynUser, epicsInt32 *value)
{
    extern int rtems_bsp_cpu_load_percentage(void);

    *value = rtems_bsp_cpu_load_percentage();
    return asynSuccess;
}
static asynInt32 int32Methods = { NULL, int32Read };

/*
 * Register ourself with ASYN
 */
static void
cpuLoadDriverRegistrar(void)
{
    drvPvt *pdrvPvt;
    const char *portName = "cpuLoad";

    pdrvPvt = callocMustSucceed(sizeof(drvPvt), 1, "cpuLoad");
    if (pasynManager->registerPort(portName, 0, 1, 0, 0) != asynSuccess) {
        printf("registerPort failed\n");
        return;
    }
    pdrvPvt->common.interfaceType = asynCommonType;
    pdrvPvt->common.pinterface  = &commonMethods;
    pdrvPvt->common.drvPvt = pdrvPvt;
    if (pasynManager->registerInterface(portName, &pdrvPvt->common) != asynSuccess) {
        printf("registerInterface failed\n");
        return;
    }
    pdrvPvt->int32.interfaceType = asynInt32Type;
    pdrvPvt->int32.pinterface  = &int32Methods;
    pdrvPvt->int32.drvPvt = pdrvPvt;
    if (pasynManager->registerInterface(portName, &pdrvPvt->int32) != asynSuccess) {
        printf("registerInterface failed\n");
        return;
    }
}
epicsExportRegistrar(cpuLoadDriverRegistrar);
